// File: applib.h
//
// Description: Header file for the OS/2 Presentation
//              Manager class library.
//
// Copyright 1994 by Mark Watson Associates
//
//       No binary rights reserved: this software library
//       may be used in compiled form without restrictions.
//       All source rights reserved: Source code to the
//       GUI library can not be distributed (on bulletin
//       boards, or as part of shareware or commercial
//       products) without written permission.
//
//       This software is provided "as is". The user
//       accepts all responsibility for its use.
//
// Note: Most windowing systems allow the allocation of
//       "user data" to a window.  I use this data to hold
//       a pointer to the C++ TAppWindow object that 
//       corresponds to the physical window.  This allows
//       me to access the C++ TAppWindow object from the
//       'C' language Window callback function.
//       Presentation Manager windows don't have this
//       extra data attribute, so I use public static
//       class storage in class Application to hold and
//       provide access to the SINGLE TAppWindow object
//       that is contained by an instance of the Application
//       class.  This should not cause any special problems
//       since any program that you write can only contain 
//       a single instance of each of the classes Application
//       and TAppWindow.
//


#ifndef applib_h
#define applib_h

#define INCL_WIN
#define INCL_WINDIALOGS
#define INCL_GPI
#define INCL_WINMENUS
#define INCL_WINMESSAGEMGR
#define INCL_DOSPROCESS

#include <os2.h>

// The following code is compiled for C++, but not
// the resource compiler (the C++ compiler defines
// '__cplusplus'):

#ifdef __cplusplus

#include <string.h>
#include <iostream.h>

extern "C" { void sprintf(...); void exit(int); };

#define output (ios::out)
#define input (ios::in)

MRESULT EXPENTRY window_func(HWND, ULONG, MPARAM, MPARAM);
MRESULT EXPENTRY dialog_func(HWND, ULONG, MPARAM, MPARAM);
VOID DisplayMessage(PCH str);

// The next declaration allows forward references in
// definition of class Application:
class TAppWindow;

class Application
{
  public:
    Application(const char *class_name,
                char *menu_name = "Menu",
                int num_menu_items = 0,
                char ** menu_items = (char **)NULL);
    int processEventMessages(void);

    enum {DIALOG_TEXT_LENGTH=128, DIALOG_MAX_LIST_ITEMS=64};

    HAB * get_HAB()                    { return hand_ab; }
    HMQ * get_HMQ()                    { return hand_mq; }
    QMSG * get_QMSG()                  { return q_mess; }
    HWND * get_HWND()                  { return hand_frame; }

    // class public member function is visible to all C
    // functions in our program:
    static TAppWindow * get_TAppWindow() { return app_window; }

    void set_HAB(HAB *a_HAB)             { hand_ab = a_HAB; }
    void set_HMQ(HMQ *a_HMQ)             { hand_mq = a_HMQ; }
    void set_QMSG(QMSG *a_QMSG)          { q_mess = a_QMSG; }
    void set_HWND(HWND *hw)              { hand_frame = hw; }

    static void set_TAppWindow(TAppWindow *aw)
    { app_window = aw; }

    // Class public variables for holding the result of Dialog
    // box functions:
    static CHAR text_from_dialog[DIALOG_TEXT_LENGTH];
    static int selection_from_dialog;
    static CHAR prompt_for_dialog[DIALOG_TEXT_LENGTH];
    static CHAR file_extension_for_dialog[DIALOG_TEXT_LENGTH];
    static CHAR * list_items_for_dialog[DIALOG_MAX_LIST_ITEMS];
    static int number_of_dialog_list_items;

  private:
    HAB *hand_ab;
    HMQ *hand_mq;
    QMSG *q_mess;
    HWND *hand_frame;

    const char * class_name;
    // All instances of this class share the following,
    // so only create one instance of an Application
    // object in your programs.  Use the following in the
    // 'C' language window callback function to access
    // our C++ Application object and C++ TAppWindow object:
    static TAppWindow * app_window;
};

const int MAX_TEXT_LINES = 24;
const int MAX_TEXT_LINE_SIZE = 80;

class TAppWindow
{
  public:
    TAppWindow(Application * my_application = NULL);

    void plot_line(int x1, int y1, int x2, int y2);
    void plot_string(int x, int y, char *cp);
    void plot_rect(int top, int right, int bottom, int left);
    void erase_rect(int top, int right, int bottom, int left);
    void redraw_display();

	MRESULT EXPENTRY WndProc(HWND hWnd, UINT iMessage,
                             MPARAM param1, MPARAM param2);

	int string_height(char *)
    {
         return 14;
	}
	int string_width(char *str)
	{
	     return 10 * strlen(str);
    }

	void clear_display();

	// user defined callback(s):
	void update_display();
	void mouse_down(int, int );
	void mouse_up(int, int );
	void mouse_move(int, int );
	void do_menu_action(int);

	// Dialog box specific member functions:
	void do_about();
	void do_edit(char *prompt, char *str);
    void show_info(char *str) { do_edit(" ", str); }
	int choose_one_from_list(char *prompt,
				 char **list_items, int number_of_items);
    int choose_one(char *prompt1, char *prompt2);
    int choose_file_to_write(char *prompt,
                             char *returned_filename);
    int choose_file_to_read(char *prompt,
                            char *extension,
                            char *returned_filename);

	void init_scrolling_text(int top,
                             int right, 
                             int bottom,
                             int left);
	void init_scrolling_text();

	void put_scrolling_text(char *str);
	void reset_scrolling_text();
    void idle_proc();
    HWND * get_HWND() { return  p_hWnd; }

  private:
	char windowName[20];
    Application * my_owning_application;
	HWND * p_hWnd;

    int file_chooser_helper(char *prompt,
                            char *extension,
                            char *returned_filename);
	// Data for supporting scrolling text:
	short top_text_clip, right_text_clip;
    short bottom_text_clip, left_text_clip;
	char *saved_text[MAX_TEXT_LINES]; // ring buffer for
                                      // saved text
    int number_of_saved_lines;
	int current_ring_buffer_start;
	int in_scrolling_text_mode; // =1 for in text mode
	int redraw_both_text_and_graphics;

	BOOL Show( int showWindowFlag );
	void Update( void );

 	void Paint( void );
	// Maintain a valid HPS only during paint operations:
    HPS hps;
    POINTL pt;
    RECTL rc;

};


void Warning(char *);

#define INIT_PROGRAM(menu_name,num,items) \
void main() \
{ \
  Application my_app("my_app", menu_name, num, items); \
  TAppWindow  my_window(&my_app);

#define RUN_PROGRAM \
	my_app.processEventMessages();

//
//  End of #ifdef _cplusplus test:
//

#endif

// Define resource specifice constants (note we use
// #define instead of the superior "const int" form
// so the resource compiler won't choke):

#define MENU1 1
#define ABOUT_DIALOG 1
#define EDIT_DIALOG  2
#define LIST_DIALOG  3
#define FILE_IN_DIALOG  4
#define FILE_OUT_DIALOG 5

#define DIALOG_OK        1002
#define DIALOG_EDIT_TEXT  101
#define DIALOG_LIST_ITEM  102

#endif
